// Copyright 2022 schukai GmbH. All rights reserved.
// Use of this source code is governed by a AGPL-3.0
// license that can be found in the LICENSE file.

package xflags

import (
	"bytes"
	"flag"
	"io"
	"reflect"
)

// New creates a new instance of the settings.
// name should be the name of the command and comes from the first argument of the command line.
// os.Args[0] is a good choice.
func New[C any](name string, definitions C) *Settings[C] {

	s := &Settings[C]{
		config: config{
			errorHandling: flag.ContinueOnError,
		},
	}

	if reflect.TypeOf(definitions).Kind() != reflect.Struct {
		s.errors = append(s.errors, newUnsupportedReflectKindError(reflect.TypeOf(definitions)))
		return s
	}

	buf := bytes.NewBufferString("")
	s.flagOutput = io.Writer(buf)
	s.definitions = definitions
	s.initCommands(name)

	return s
}

// FlagOutput returns the writer where the flag package writes its output.
func (s *Settings[C]) FlagOutput() string {
	return s.flagOutput.(*bytes.Buffer).String()
}

// Args Returns not parsed arguments.
func (s *Settings[C]) Args() []string {
	return s.args
}
