// Copyright 2022 schukai GmbH. All rights reserved.
// Use of this source code is governed by a AGPL-3.0
// license that can be found in the LICENSE file.

package xflags

import (
	"github.com/stretchr/testify/assert"
	"testing"
)

func TestNewIntWithError(t *testing.T) {

	// two is not a struct
	commands := New("root", 2)
	assert.True(t, commands.HasErrors())

}

func TestNew(t *testing.T) {

	c := struct {
		Verbose bool `short:"v" long:"verbose" description:"Show verbose debug information"`
	}{
		Verbose: false,
	}

	commands := New("root", c)
	assert.False(t, commands.HasErrors())

}

type CmdTest1 struct {
	A    bool `short:"a"`
	Sub1 struct {
		B    bool `short:"b"`
		Sub2 struct {
			C    bool `short:"c"`
			Sub3 struct {
				D bool `short:"d"`
			} `command:"sub3"`
		} `command:"sub2"`
	} `command:"sub1"`
	aa int `short:"x"`
}

func TestCommand2(t *testing.T) {

	commands := New("root", CmdTest1{})
	args := []string{"-a", "sub1", "-b", "sub2", "-c", "sub3", "-d"}

	commands.Parse(args)

	assert.False(t, commands.HasErrors())
	if commands.HasErrors() {
		t.Log(commands.Errors())
	}

	assert.True(t, CmdTest1(commands.GetValues()).A)
	assert.True(t, CmdTest1(commands.GetValues()).Sub1.B)
	assert.True(t, CmdTest1(commands.GetValues()).Sub1.Sub2.C)
	assert.True(t, CmdTest1(commands.GetValues()).Sub1.Sub2.Sub3.D)

	assert.False(t, commands.HasErrors())

}
