// Copyright 2022 schukai GmbH. All rights reserved.
// Use of this source code is governed by a AGPL-3.0
// license that can be found in the LICENSE file.

package xflags

import "os"

// ParseOsArgs parses the os.Args.
func (s *Settings[C]) ParseOsArgs() *Settings[C] {
	return s.Parse(os.Args[1:])
}

// Parse parses the command line arguments and assigns the values to the settings.
func (s *Settings[C]) Parse(args []string) *Settings[C] {
	if len(s.errors) > 0 {
		return s
	}

	if s.command == nil {
		s.errors = append(s.errors, MissingCommandError)
		return s
	}

	err := s.command.flagSet.Parse(args)
	if err != nil {
		s.errors = append(s.errors, err)
		return s
	}

	s.assignValues(*s.command)

	r := s.command.flagSet.Args()
	if len(r) == 0 {
		return s
	}

	s.command.parse(r)

	return s
}
