// Copyright 2022 schukai GmbH. All rights reserved.
// Use of this source code is governed by a AGPL-3.0
// license that can be found in the LICENSE file.

package xflags

import (
	"reflect"
	"strconv"
)

const (
	tagIgnore      = "ignore"
	tagCall        = "call"
	tagCommand     = "command"
	tagShort       = "short"
	tagLong        = "long"
	tagDescription = "description"
	tagShadow      = "shadow"
)

func getTagMap(field reflect.StructField) (value map[string]string) {

	tagValues := map[string]string{}
	tag := field.Tag

	// code from reflect.StructTag.Lookup
	for tag != "" {
		// Skip leading space.
		i := 0
		for i < len(tag) && tag[i] == ' ' {
			i++
		}
		tag = tag[i:]
		if tag == "" {
			break
		}

		i = 0
		for i < len(tag) && tag[i] > ' ' && tag[i] != ':' && tag[i] != '"' && tag[i] != 0x7f {
			i++
		}
		if i == 0 || i+1 >= len(tag) || tag[i] != ':' || tag[i+1] != '"' {
			break
		}
		name := string(tag[:i])
		tag = tag[i+1:]

		// Scan quoted string to find value.
		i = 1
		for i < len(tag) && tag[i] != '"' {
			if tag[i] == '\\' {
				i++
			}
			i++
		}
		if i >= len(tag) {
			break
		}
		qvalue := string(tag[:i+1])
		tag = tag[i+1:]

		value, err := strconv.Unquote(qvalue)
		if err != nil {
			break
		}

		tagValues[name] = value

	}

	return tagValues
}
